#!/usr/bin/perl -w
# TRAP version 1.1.0 - 17 January 2007
use strict;
use warnings;
use Getopt::Long;

#
#input parameters 
#
my $conf;
my $input_prefix = "";
my $output_file_prefix = "TRAP_file";
my $output_directory = "TRAP_dir";
my $minimum_period_size = 1;
my $maximum_period_size = 2000;#maximum size accepted by TRF
my $minimum_copy_number = 2;
my $maximum_copy_number = 1000000000;#just a very big number
my $minimum_flanking_region_size = 0;
my $define_repeat_sequence = "";
my $minimum_match_percentage = 0;
my $create_flanking_region_file;
my $create_consensus_file;
my $create_trf_like_file;
my $create_feature_table;
my $create_feature_table_artemis;
my $create_gff;
my $create_redundancy_report;
my $table_format;
my $sort_field;
my $help;

#
#Global variables to control number of repeats
#(some of them help multi-file output handling)
#

my $total_selected_repeats = 0;
my $current_sequence_selected_repeats = 0; 
my $next_output_file_number = 1;
my $txt_file_header;
my @positions_start;
my @positions_end;
my @positions;
my $html_data_string;
my $txt_data_string;
my $fasta_number;

#
#global variables to define the size of entries in HTML tables
#
my $max_entries_output_tables = 25;

#
#auxiliary variables linux/windows
#
my $dir_separator =  "\/";

#
#First get command-line parameters
#

GetOptions( "c=s"      => \$conf,
	    "i=s"      => \$input_prefix,
	    "od=s"     => \$output_directory,
	    "of=s"     => \$output_file_prefix,

	    "min=s"    => \$minimum_period_size,
	    "max=s"    => \$maximum_period_size,
	    "cpmin=s"  => \$minimum_copy_number,
	    "cpmax=s"  => \$maximum_copy_number,
	    "fs=s"     => \$minimum_flanking_region_size,
	    "seq=s"    => \$define_repeat_sequence,
	    "id=s"     => \$minimum_match_percentage,

	    "ff"       => \$create_flanking_region_file,
	    "trf"      => \$create_trf_like_file,
	    "con"      => \$create_consensus_file,
	    "ft"       => \$create_feature_table,
	    "aft"      => \$create_feature_table_artemis,
	    "gff"      => \$create_gff,
	    "rr"       => \$create_redundancy_report,
	    
	    "tbf=s"    => \$table_format,
	    "sort=s"   => \$sort_field,
	    
	    "help"     => \$help
	    );

my $help_print = "./TRAP.pl\n
TRAP version 1.1.0 - 7 January 2007

-i          [input file]

-od         [output directory]: default = TRAP_dir#
-of         [output file]: default = TRAP_file#


-min        [minimum period size of repeats]
-max        [maximum period size of repeats]
-cpmin      [minimum copy number]
-cpmax      [maximum copy number]
-fs         [minimum length of flanking regions]
-id         [minimum % of identity]
-seq        [sequence of desired unit]


-ff         [create flanking file]
-trf        [create TRF-like HTML files]
-con        [show consensus sequence]
-ft         [create feature table]
-aft        [create ARTEMIS feature table]
-gff        [create gff file(s)]
-rr         [create redundancy report file]


-tbf        [html - default parameter OR
             csv OR
             html+csv]

-sort       [bases   - (Total number of bases) - default parameter OR
             loci    - (Total number of repeat loci) OR
             units   - (Total number of repeat units) OR
             average - (Average number of repeat units per locus) OR
             size    - (Period size)]

-c          [configuration file]

-help       [show this parameter list]\n
";


if ($create_flanking_region_file){
    $create_flanking_region_file = "yes";
}
else{
$create_flanking_region_file = "no";
}
if ($create_consensus_file){
    $create_consensus_file = "yes";
}
else{
    $create_consensus_file = "no";
}
if ($create_trf_like_file){
    $create_trf_like_file = "yes";
}
else{
    $create_trf_like_file = "no";
}
if ($create_feature_table){
    $create_feature_table = "yes";
}
else{
    $create_feature_table = "no";
}
if ($create_feature_table_artemis){
    $create_feature_table_artemis = "yes";
}
else{
    $create_feature_table_artemis = "no";
}
if ($create_gff){
    $create_gff = "yes";
}
else{
    $create_gff = "no";
}
if ($create_redundancy_report){
    $create_redundancy_report = "yes";
}
else{
    $create_redundancy_report = "no";
}

#
#if user specifies configuration file, the configuration file data will override
#command line specifications
#
if ($conf){
    print STDERR "Configuration file specified, command line options will be overridden\n\n";

    $input_prefix = "";
    $output_file_prefix = "TRAP_file";
    $output_directory = "TRAP_dir";

    $minimum_period_size=1;
    $maximum_period_size=2000;#maximum size accepted by TRF
    $minimum_copy_number=2;
    $maximum_copy_number=100000000;#just a very big number
    $minimum_flanking_region_size=1;
    $define_repeat_sequence = "";
    $minimum_match_percentage = "";
    $create_flanking_region_file = "";
    $create_consensus_file = "";
    $create_trf_like_file = "";
    $create_feature_table = "";
    $create_feature_table_artemis = "";
    $create_gff = "";
    $create_redundancy_report = "";
    $table_format = "";
    $sort_field = "";
    $help = "";
    
    #
    #auxiliary variables
    #
    my @arguments;
    my %config;
    my $missingArgument=0;

    #
    #read configuration file
    #
   
    open(CONFIG, "$conf") or die("missing configuration file $conf\n");

    my $configLine;
    while ($configLine = <CONFIG>){
	$configLine =~ s/^\s+//;
	$configLine =~ s/\s+\Z//;
	$configLine =~ s/\s+\=/\=/;
	$configLine =~ s/\=\s+/\=/;
	if ($configLine =~ /^\#/ || !($configLine =~ /(.)\=(.)/)) {
	    next;
	}
	chomp $configLine;
	@arguments = split("=", $configLine);
	$config{$arguments[0]}=$arguments[1];
    }
    
    close(CONFIG);
     
    #
    #now check if any mandatory argument is missing
    #

    if (!exists($config{"input_prefix"})){
	$missingArgument = 1;
	print "Missing mandatory configuration argument:input_prefix\n";
    }
    else {$input_prefix = $config{"input_prefix"}};
    if ($missingArgument) {
	die "\n\nCannot run program, mandatory configuration argument missing from configuration file (see above)\n";
    }
    #set optional arguments that were declared in configuration file

    if (defined($config{"output_file_prefix"})){
       $output_file_prefix = $config{"output_file_prefix"};
    }
    if (defined($config{"output_directory"})){
       $output_directory = $config{"output_directory"};
    }
    if (defined($config{"minimum_period_size"})){
       $minimum_period_size = $config{"minimum_period_size"};
    }
    if (defined($config{"maximum_period_size"})){
       $maximum_period_size = $config{"maximum_period_size"};
    }
    if (defined($config{"minimum_copy_number"})){
       $minimum_copy_number = $config{"minimum_copy_number"};
    }
    if (defined($config{"maximum_copy_number"})){
       $maximum_copy_number = $config{"maximum_copy_number"};
    }
    if (defined($config{"minimum_flanking_region_size"})){
       $minimum_flanking_region_size = $config{"minimum_flanking_region_size"};
    }
    if (defined($config{"define_repeat_sequence"})){
       $define_repeat_sequence = $config{"define_repeat_sequence"};
    }
    if (defined($config{"minimum_match_percentage"})){
       $minimum_match_percentage = $config{"minimum_match_percentage"};
    }
    if (defined($config{"create_flanking_region_file"})){
       $create_flanking_region_file = $config{"create_flanking_region_file"};
    }
    if (defined($config{"create_consensus_file"})){
       $create_consensus_file = $config{"create_consensus_file"};
    }
    if (defined($config{"create_trf_like_file"})){
       $create_trf_like_file = $config{"create_trf_like_file"};
    }
    if (defined($config{"create_feature_table"})){
       $create_feature_table = $config{"create_feature_table"};
    }
    if (defined($config{"create_feature_table_artemis"})){
       $create_feature_table = $config{"create_feature_table_artemis"};
    }
    if (defined($config{"create_gff"})){
       $create_gff = $config{"create_gff"};
    }
    if (defined($config{"create_redundancy_report"})){
       $create_redundancy_report = $config{"create_redundancy_report"};
    }
    if (defined($config{"table_format"})){
       $table_format = $config{"table_format"};
    }
    if (defined($config{"sort_field"})){
       $sort_field = $config{"sort_field"};
    }

    $create_flanking_region_file = lc $create_flanking_region_file;
    $create_consensus_file = lc $create_consensus_file;
    $create_trf_like_file = lc $create_trf_like_file;
    $create_feature_table = lc $create_feature_table;
    $create_feature_table_artemis = lc $create_feature_table_artemis;
    $create_gff = lc $create_gff;
    $create_redundancy_report = lc $create_redundancy_report;

}

#
#help 
#

if ($help){
    print "$help_print";
    exit;
}

#
#for easy debugging, we should always flush buffers
#
$| = 1;

#
#INPUT PARAMETER VERIFICATION
#


#
#default sort field is total number of bases
#in all repetitive regions of the pattern
#

if (($table_format) && (!$sort_field)){
    $sort_field = "bases";
    $create_trf_like_file = "yes";
}

if ((!$table_format) && ($sort_field)){
    $table_format = "html";
    $create_trf_like_file = "yes";
}

#
#number of repeat units
#
if ($minimum_copy_number < 2){
    die "Minimum number of repeat units  has to be greater than 2\n";
}

if ($maximum_copy_number < 2){
    die "Maximal number of repeat units has to be greater than 2\n";
}
if ($minimum_copy_number > $maximum_copy_number){
    die "Maximum number of repeat units has to be greater than minimum number of repeat units \n";
}

#
#period size
#

if ($minimum_period_size <= 0){
    die "Minimum period size of the repeat unit has to be greater than 0\n";
    exit;
}
if ($minimum_period_size >= 2000){
    die "Minimum repeat period has to be smaller than 2000\n";
    exit;
}
if ($maximum_period_size <= 0){
    die  "Maximum period size of the repeat unit has to be greater than 0\n";
}
if ($maximum_period_size > 2000){
    die  "Maximum period size of the repeat unit has to be smaller than 2001\n";
}
if ($minimum_period_size > $maximum_period_size){
    die "Maximum period size of the repeat unit has to be larger than or equal of minimum period size\n";
}

#
#flanking region size
#
if ($minimum_flanking_region_size > 500){
    die "Maximum flanking sequence 500 bp\n";
}

#
#inconsistent parameter settings
#

if ((!$table_format) && ($sort_field)){
    die "Table format is not defined\n";
}

if (($table_format) || ($sort_field)){

    $table_format = lc($table_format);
    $sort_field = lc($sort_field);
    
    if (($table_format !~ /html/) && ($table_format !~ /csv/)){
	die "Table format option not available\n";
    }
    if ($table_format =~ /\+/){
	if (($table_format !~ /html\+csv/) && ($table_format !~ /csv\+html/)){
	    die "Table format option not available\n";
	}
    }

    if (($sort_field !~ /size/) && ($sort_field !~ /bases/) && ($sort_field !~ /loci/) && ($sort_field !~ /units/) && ($sort_field !~ /average/)) {
      die "Table sorting option not avaliable\n Available sorting options: size, bases, loci, units or average\n";
   
    }
}

#
#END OF INPUT PARAMETER VERIFICATION
#

#
#get name of the files to be opened
#

my @input_files;
my $dir_input = "";

if ($input_prefix ne ""){

    #
    #there are many possibilities for file name formats and placements
    #

    #
    #file not in current directory with .txt.html suffix
    #
    
    if ($input_prefix =~ m/([.\w\d\/]+\/)(.*.txt.html)$/){

	$dir_input = $1;
	my $file = $2;
	
	opendir DIR,"$dir_input";
	@input_files = grep(/$file/,readdir(DIR));
	close DIR;
    }
    #
    #file not in current directory with .html extension only
    #
    elsif ($input_prefix =~ m/([.\w\d\/]+\/)(.*).html$/){
	$dir_input = $1;
	my $file = $2;

	opendir DIR,"$dir_input";
	@input_files = grep(/$file.txt.html/,readdir(DIR));
	close DIR;
    }
    #
    #file in current directory with .txt.html suffix
    #
    elsif ($input_prefix =~ m/.*.txt.html$/){
	opendir DIR,".";
	@input_files = grep(/$input_prefix/,readdir(DIR));
	close DIR;
    }
    #
    #file in current directory with .html suffix only
    #
    elsif ($input_prefix =~ m/(.*).html$/){
	my $file = $1;
	
	opendir DIR,".";
	@input_files = grep(/$file.txt.html/,readdir(DIR));
	close DIR;
    }
    #
    #file not in current directory with free suffix
    #
    elsif ($input_prefix =~ m/([.\w\d\/]+\/)(.*)$/){
	$dir_input = $1;
	my $file = $2;
	
	opendir DIR,"$dir_input";
	@input_files = grep(/$file.*txt.html/,readdir(DIR));
	close DIR;
    }
    #
    #file in current directory with free suffix
    #
    else{
	opendir DIR,".";
	@input_files = grep(/$input_prefix.*txt.html/,readdir(DIR));
	close DIR;
    }
}
else {
    die "\n\nYou should define the input file name \n\n\n $help_print";
}

#
#is there any file to be processed?
#
if (@input_files+0 <= 0){
  die "input file name not found\n\n";
}

#
#If user does not specify a directory, TRAP uses a default name
#Existing directories are not overwritten, so a suffix number is appended
#

if ($output_directory =~ m/^TRAP_dir$/){
    my $contdir = 1;
    while (-d "TRAP_dir$contdir"){
	$contdir ++;
    }
    $output_directory .= $contdir;
}

#
#create output directory
#
if (!(-d $output_directory)){
    system "mkdir $output_directory";
}


#
#If user does not specify a output prefix file name, TRAP uses a default name
#Existing file are not overwritten, so a suffix number is appended
#

if ($output_file_prefix =~ m/^TRAP_file$/){

    my $contdir = 1;

    opendir DIR,"$output_directory";
    my @input_files = grep(/TRAP_file/,readdir(DIR));
    close DIR;

    my $number_old = 0;

    foreach my $aux (@input_files){
	if ($aux =~ m/TRAP_file(\d+)/){
	    if ($1 > $number_old){
		$number_old = $1;
	    }
	}

    }

    $output_file_prefix .= $number_old+1;
}


#
#complete the output file path
#
my $output_prefix = "$output_directory$dir_separator$output_file_prefix";

#
#Because the repetitive locus can start on any base of the repeat unit,
#when a specific repeat sequence is requested, we need to check all
#possibilities (e.g. for "ATG" we generate also "TGA" and "GAT") and also
#their corresponding reverse complement sequences
#
my %repeat_variations;
if ($define_repeat_sequence){

    $define_repeat_sequence = uc $define_repeat_sequence;
    %repeat_variations = compute_variations($define_repeat_sequence);
}

#
#In order to process multiple-file output from TRF we
#need to identify which are the files that correspond
#to the same TRF input. When this happens, we have files
#with the same prefix, a number, and the suffix ".txt.html".
#These files need to be processed in ascending order of the number,
#therefore one need to sort the file names.
#

my %names_files;
my @number_files;

#
#extracting the numbers and storing file names associated with them
#
               
foreach my $aux (@input_files){

    #
    #TRF ouputs at least one file per sequence, sometimes more than one
    #file per input sequence. We need to extract, from each file name,
    #the sequence number (if there is more than one), and we need to
    #identify the files referring to the same sequence
    #

    if ($aux =~ m/\.s(\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.(\d+)\.txt.html/){
	my $aux1 = "$1.$2";
	push @number_files, $aux1;
	$names_files{$aux1} = $aux;
    }
    else{
	$aux =~ m/\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.(\d+)\.txt.html/;
	my $aux1 = "1.$1";
	push @number_files, $aux1;
	$names_files{$aux1} = $aux;
    }
}

#
#sorting the file numbers
#

@number_files = sort { 
    $a =~ m/(\d+).(\d+)/;
    my @c;
    push @c,$1,$2;
    $b =~ m/(\d+).(\d+)/;
    my @d;
    push @d,$1,$2;
    
    $c[0] <=> $d[0]
	||
    $c[1] <=> $d[1]
		
} @number_files;

#
#extract each file in order and refill @input_files
#

@input_files = ();

foreach my $aux (@number_files){
    push @input_files, $names_files{$aux};
}

#
#open output files
#

if ($create_consensus_file eq "yes"){
    open (CONSENSUS,">$output_prefix"."_consensus.txt") 
	or die "cannot open $output_prefix"."consensus.txt";
}

if ($create_flanking_region_file eq "yes") {
    open (FLANK_SEQ,">$output_prefix"."_flanking_regions.txt") 
	or die "cannot open $output_prefix"."_flanking_regions.txt";
}

#
#create directory to store "feature table" output
#
if ($create_feature_table eq "yes"){

    chdir "$output_directory";
    if (!(-d "feature_table")){
	system "mkdir feature_table";
    }
    chdir "..";
}

#
#create directory to store "ARTEMIS feature table" output
#
if ($create_feature_table_artemis eq "yes"){

    chdir "$output_directory";
    if (!(-d "feature_table_artemis")){
	system "mkdir feature_table_artemis";
    }
    chdir "..";
}
#
#create directory to store ggf files output
#
if ($create_gff eq "yes"){

    chdir "$output_directory";
    if (!(-d "gff")){
	system "mkdir gff";
    }
    chdir "..";
}


#
#create directory to store html output
#
if (($create_trf_like_file eq "yes") || ($table_format) || ($create_redundancy_report eq "yes")){

    chdir "$output_directory";
    if (!(-d "html_data")){
	system "mkdir html_data";
    }
    chdir "..";
}

#
#Auxiliary variables to compute statistics
#

my $copy_number_sum = 0;
my $current_file_repeats = 0;
my @file_summary;
my $first_time = 0;
my %length_counter;
my $next_repeat_code = 0;
my $original_file_prefix;
my @repeat_code_representative;
my $sequence_name;
my @size_ms;
my @sum_of_sizes_for_repeat;
my $total_selected_bases = 0;
my @total_copies_per_repeat;
my @total_number_of_repetitive_regions;
my %total_number_of_copies_size;
my $multi_file_output;
my $input_file_prefix;
my $input_file_prefix_old = "";
my %repeat_code;
my $trf_parameters;	
my $feature_table_file = "";
my $feature_table_file_artemis = "";
my $gff_table_file = "";
my $sequence_name_number_sequence_ft = "";
my $sequence_name_number_sequence_gff = "";

my $repeat_start = "";
my $repeat_end = "";
my $total_size_of_repeat_regions = 0;
my @repeat_region_list;
my @redundant_repeat_summary_file;

my @html_page_repeat_group;
#
#PROCESS ONE FILE AT A TIME
#

foreach my $input_file (@input_files){

    my $consensus_sequence;
    #my $sequence_size;
    my $file_header;

    
    #
    #TRF always outputs files with the suffix ".txt.html".
    #Multiple files originated from the same sequence
    #have the same prefix, a ".", and a number preceding the
    #suffix.
    #
    $input_file =~ m/(.*)\.\d+\.txt.html/;
    $input_file_prefix = $1;
 
    #
    #Compare old and new prefixes to check if we are in the middle
    #of a multifile output
    #

    if ($input_file_prefix ne $input_file_prefix_old){
	#
	#not part of a multifile output, not after a multifile output
	#	
	$multi_file_output = 0;
    }
    else{
	#
	#part of a multifile output
	#
	$multi_file_output = 1;
    }

    #
    #if HTML tables are requested and we have some data
    #

    if (($create_trf_like_file eq "yes") && ($positions_start[0])){
	if ((!$multi_file_output) && ($first_time == 1) ){
	    #
	    #get last repeat's start and end
	    #
	    my $index_start = $positions_start[0];
	    my $index_end = pop @positions_end;
	    push @positions, "$index_start to $index_end";
	    #
	    #print stuff
	    #
	    my $is_last = 1;
	    print_files($is_last, $next_output_file_number,$txt_file_header);
	    
	    #
	    #update summary
	    #

	    #
	    #add link
	    #
	    my $summary_file_name = "html_data\/$output_file_prefix\_TRAP_$original_file_prefix";
	    
	    if ($next_output_file_number > 1){
		print_html_index($output_directory,"$output_file_prefix\_TRAP_$original_file_prefix",\@positions);
		$summary_file_name .= "_index.html"; 
	    }
	    else{
		$summary_file_name  .= ".1.html";
	    }
	    #
	    #update summary
	    #
	    my $summary_entry =  "<TR>".
		"<TD BGCOLOR=\"\#FFCC66\"><CENTER>".$fasta_number."<\/CENTER><\/TD>".
		"<TD><CENTER><A TARGET=\"$summary_file_name\" HREF=\"$summary_file_name\">".$sequence_name."<\/A><\/CENTER><\/TD>".
		"<TD BGCOLOR=\"\#FFCC66\"><CENTER>".$current_sequence_selected_repeats."<\/CENTER><\/TD><\/TR>\n";
	    
	    push @file_summary, $summary_entry;

	    #
	    #We have finished a sequence, need to re-initialize
	    #global variables
	    #

	    $current_file_repeats = -1;
	    $next_output_file_number = 1;
	    $current_sequence_selected_repeats = 0;
	    @positions_start = ();
	    @positions_end = ();
	    @positions = ();
	}	
    }

    #
    #feature table data is generated only at the end or processing, independently of
    #the number of html pages 
    #
    if (($create_feature_table eq "yes")  && ($feature_table_file) && ($first_time == 1) && ($multi_file_output == 0)){
	my $feature_file;   

	if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	    $feature_file = $1;   
	}
	else{
	    $original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	    $feature_file = $1;    
	}

	#
	#use the first 20 characters of the sequence name to name the .tab file
	#(spaces and other special characters are replaced by underlines)
	#

	my $sequence_name_aux = substr($sequence_name,0,25);

	$sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
	$sequence_name_aux =~ s/\s/_/;
	$sequence_name_aux =~ s/\t/_/;

	$sequence_name_number_sequence_ft .= "$sequence_name;$output_file_prefix\_$feature_file\_$sequence_name_aux.tab\n";

	open (FEATURE_TABLE_FILE,">$output_directory/feature_table/$output_file_prefix\_$feature_file\_$sequence_name_aux.tab");
	print FEATURE_TABLE_FILE "$feature_table_file\n";

	close (FEATURE_TABLE_FILE);
	$feature_table_file = "";
    }

#
    #ARTEMIS feature table data is generated only at the end or processing, independently of
    #the number of html pages 
    #
    if (($create_feature_table_artemis eq "yes")  && ($feature_table_file_artemis) && ($first_time == 1) && ($multi_file_output == 0)){
	my $feature_file;   

	if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	    $feature_file = $1;   
	}
	else{
	    $original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	    $feature_file = $1;    
	}

	#
	#use the first 20 characters of the sequence name to name the .tab file
	#(spaces and other special characters are replaced by underlines)
	#

	my $sequence_name_aux = substr($sequence_name,0,25);

	$sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
	$sequence_name_aux =~ s/\s/_/;
	$sequence_name_aux =~ s/\t/_/;

	$sequence_name_number_sequence_ft .= "$sequence_name;$output_file_prefix\_$feature_file\_$sequence_name_aux.tab\n";

	open (FEATURE_TABLE_FILE_ARTEMIS,">$output_directory/feature_table_artemis/$output_file_prefix\_$feature_file\_$sequence_name_aux.tab");
	print FEATURE_TABLE_FILE_ARTEMIS "$feature_table_file_artemis\n";

	close (FEATURE_TABLE_FILE_ARTEMIS);
	$feature_table_file_artemis = "";
    } 


    #
    #gff data is generated only at the end or processing, independently of
    #the number of html pages 
    #
    if (($create_gff eq "yes")  && ($gff_table_file) && ($first_time == 1) && ($multi_file_output == 0)){
	my $gff_file;   

	if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	    $gff_file = $1;   
	}
	else{
	    $original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	    $gff_file = $1;    
	}

	#
	#use the first 20 characters of the sequence name to name the .gff file
	#(spaces and other special characters are replaced by underlines)
	#

	my $sequence_name_aux = substr($sequence_name,0,25);

	$sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
	$sequence_name_aux =~ s/\s/_/;
	$sequence_name_aux =~ s/\t/_/;

	$sequence_name_number_sequence_gff .= "$sequence_name;$output_file_prefix\_$gff_file\_$sequence_name_aux.gff\n";

	open (GFF_TABLE_FILE,">$output_directory/gff/$output_file_prefix\_$gff_file\_$sequence_name_aux.gff");
	print GFF_TABLE_FILE "$gff_table_file\n";

	close (GFF_TABLE_FILE);
	$gff_table_file = "";
    } 


    #
    #compute redundancy
    #

  
    if ( (($create_redundancy_report eq "yes") || ($sort_field)) && (($multi_file_output == 0) && ($first_time == 1)) ){

	$total_size_of_repeat_regions += repeat_regions_analysis (\@repeat_region_list,$create_redundancy_report, $input_file_prefix_old);
	
	@repeat_region_list = ();
    }
	
	
    #
    #Since we can have multiple file input, we need to generate output 
    #only AFTER we are sure there are no more files for the same input.
    #Thus, we keep track of input file name prefixes.
    #
    $input_file_prefix_old = $input_file_prefix;

    $first_time = 1;

    #
    #TRF OUTPUT analysis
    #
    open(TRFDATA, "$dir_input$input_file") 
	or die "cannot open $input_file";
    #
    #Remove path from file name
    #
    if ($input_file =~ m/[.*\/]*.*\/(.*).txt.html/){
	$original_file_prefix = $1;
    }
    elsif ($input_file =~ m/(.*).txt.html/){
	$original_file_prefix = $1;
    }  

    #TRF ouputs at least one file per sequence, sometimes more than one
    #file per input sequence. We need to extract, from each file name,
    #the sequence number (if there is more than one), and we need to
    #identify the files referring to the same sequence
    #

    if ($original_file_prefix =~ m/(.*\.s(\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+)\.\d+/){
	$original_file_prefix = $1;   
	$fasta_number = $2;
    }
    else{
	$original_file_prefix =~ m/(.*\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+)\.\d+/;
	$original_file_prefix = $1;    
	$fasta_number = 1;
    }
    
    

    #
    #Read one result at a time
    #
    $/ = "\nFound at";
    
    $file_header = <TRFDATA>;

    $file_header =~ m/Sequence:\s+(.*)/;
    $sequence_name = $1;

    $file_header =~ m/Parameters:(.*)/;
    my $trf_parameters = "$1";

    $file_header =~ m/Version (.*)/;
    my $version = $1;

    #
    #First get sequence name from the beginning of the file
    #
    if ($create_trf_like_file eq "yes"){
		
	$file_header =~ m/<TITLE>(.*)<\/TITLE>/;
	my $title = $1;
	$file_header =~ m/(Parameters:.+)File/s;	
	my $parameters = $1;	

$txt_file_header  = "
<HTML><HEAD><TITLE>$title<\/TITLE><\/HEAD><BODY bgcolor=\"\#FFFFCC\"><PRE>
TRAP (Tandem Repeats Analysis Program)

TRF Version $version

Sequence: $sequence_name

$parameters

Found at i:";

    }

    #
    #Loop on each TRF result
    #

  REPEATENTRYLOOP:
    while (my $entry = <TRFDATA>){

	#
	#get HTML link into another file
	#
	$entry =~ m/(.*<A NAME\=\")(\d+\-\-\d+,.*\>)(\<\/A\>\<A HREF=\"http:\/\/.*\#alignment\" target =\"explanation\">Alignment explanation<\/A><BR><BR>)(.*)/s; 
	my $html_link = $2;
	
	#
        #updated Alignment explanation html link
	#
	$entry = $1.$2."<\/A><A HREF=\"http:\/\/tandem.bu.edu\/trf\/trf.definitions.html\#alignment\" target =\"explanation\">Alignment explanation<\/A><BR><BR>".$4;


	#
	#get indices, number of copies, consensus size
	#
	$entry =~ m/Indices: (\d+)--(\d+)\s+Score: (\d+)\n\s+Period size:\s+(\d+)\s+Copynumber:\s([\d\.]+).*Consensus size:\s+(\d+)/;
	

	$repeat_start = $1;
	$repeat_end = $2;
	my $score = $3;
	my $period = $4;
	my $copynumber = $5;
	my $consensus_size = $6;
	
	#
	#Get match identity percentage, indel percentage, nucleotide percentage
	#
	
	my  $unparsed_entry = $';                                                       #' empty comment  
	$unparsed_entry =~ m/Indels:\s*\d*.*\n\s+([\d\.]+)\s+[\d\.]+\s+([\d\.]+)/;
	my $match_percentage = $1*100;
	my $percente_indels = $2*100;
	$unparsed_entry =~ m/ACGTcount: A:([\d\.]+), C:([\d\.]+), G:([\d\.]+), T:([\d\.]+)/;
	my $adenine = $1*100;
	my $citose  = $2*100;
	my $guanine = $3*100;
	my $timine  = $4*100;
	
	#
	#consensus pattern
	#

	$unparsed_entry = $';                                                            #' empty comment
	
	$unparsed_entry =~ m/Consensus pattern .(\d+) bp.*([ACGTacgt\n]+)/;
	$unparsed_entry = $';                                                            #'empty comment    
	my $entry_consensus_size = $1;
	
	$consensus_sequence = $2;
	$consensus_sequence =~ s/\n//g;

	#
	#check if flanking regions were computed when TRF was run
	#

	if ($unparsed_entry !~ m/Left flanking sequence/){
	    if (($minimum_flanking_region_size) || ($create_flanking_region_file eq "yes")){
		die "TRF did not calculate flanking sequences, you should run TRF again\n";
	    }
	}
	#
	#Upstream flanking region
	#

	my $flanking_upstream;
	my $flanking_upstream_start;
	my $flanking_upstream_end;
	if ($unparsed_entry =~ m/Left flanking sequence:\s+Indices\s+(\d+)[\s\-]+(\d+)[\s\n]*([A-Za-z\n]+)\n/){
	    $flanking_upstream_start = $1;
	    $flanking_upstream_end = $2;
	    $flanking_upstream = $3;
	    $flanking_upstream =~ s/\n//g;
	}
	else{
	    $flanking_upstream_start = 0;
	    $flanking_upstream_end = 0;
	    $flanking_upstream = "";
	}

	#
	#Downstream flanking region
	#

	my $flanking_downstream_start;
	my $flanking_downstream_end;
	my $flanking_downstream;
	if ($unparsed_entry =~ m/Right flanking sequence:\s+Indices\s+(\d+)[\s\-]+(\d+)[\s\n]*([A-Za-z\n]+)\n/){
	    $flanking_downstream_start = $1;
	    $flanking_downstream_end = $2;
	    $flanking_downstream = $3;
	    $flanking_downstream =~ s/\n//g;
	}
	else{
	    $flanking_downstream_start = 0;
	    $flanking_downstream_end = 0;
	    $flanking_downstream = "";      
	}
	    
	#
	#Validating repeat candidate
	#
	
################################################
###########################################
#######################################
################################
#########################
################
##########
#####

	#
	#first we reduce as we can the size of the repeat
	#for this, we will try all possibilities of 
	#subsequences of my repeat sequence
	#

	#
	#the trick is to find valid sizes, which are the numbers
	#that divide my repeat size
	#
	if ($consensus_size > 1){

	    my @divisores = all_divisores($consensus_size);

	    foreach my $size (sort (@divisores)){
		#
		#for each size , we test the 
		#if the first "size" charcters repeat themselves
		#
		my $test_repeat = $consensus_sequence;
		
		my $candidate = substr($consensus_sequence,0,$size);
		if ($test_repeat =~ m/^($candidate)+$/){
		    $consensus_sequence = $candidate;
		    $consensus_size = length $consensus_sequence;

		    $copynumber = ($copynumber * (length $test_repeat))/$consensus_size ;

		    last;
		}
	    }
	}
###
#############
#########################
################################
#######################################
##########################################################
##############################################################



	#
	#1.check minimum size and maximum copy number
	#
	if ($maximum_copy_number){
	    if (int $copynumber > $maximum_copy_number){
		next REPEATENTRYLOOP;      
	    }
	}
	if ($minimum_copy_number){
	    if (int $copynumber < $minimum_copy_number){
		next REPEATENTRYLOOP;
	    }
	}
	
	#
	#2.check maximum and minimum period size
	#
	if ($maximum_period_size){
	    if ($maximum_period_size < $consensus_size){
		next REPEATENTRYLOOP;
	    }
	}
	
	if ($minimum_period_size){
	    if ($minimum_period_size > $consensus_size){
		next REPEATENTRYLOOP;
	    }
	}
	
	#
	#3. check minimum match percentage
	#

	if ($minimum_match_percentage){
	    if ($match_percentage < ($minimum_match_percentage)){
		next REPEATENTRYLOOP;
	    }
	}

	#
	#if we are looking for a specific repeat...
	#

	if ($define_repeat_sequence ne ""){
	    #
	    #Check if the repeat found is one we are looking for
	    #
	    if (!exists($repeat_variations{$consensus_sequence})){
		next REPEATENTRYLOOP;
	    }
	}
	
	#
	#if minimum flanking region has been specified
	#
	
	if ($minimum_flanking_region_size){
	    #
	    #check if upstream flanking sequence has the minimum size requested
	    #
	    if (($flanking_upstream_end - $flanking_upstream_start + 1) < $minimum_flanking_region_size){
		next REPEATENTRYLOOP;
	    }
	    
	    #
	    #check if downstream flanking sequence has the minimum size requested
	    #
	    if (($flanking_downstream_end - $flanking_downstream_start + 1) < $minimum_flanking_region_size){
		next REPEATENTRYLOOP;
	    }
	}
	
	#
	#any repeat region that reaches this point is acceptable and will be registered
	#

	#
	#generate entry in the HTML report, if it is the case
	#

	if ($create_trf_like_file eq "yes"){
	    #
	    # we need to keep track of how many entries we are putting in each file,
	    # (max is in global variable $max_entries_outpout_table)
	    #
	    $current_file_repeats++;
	    if ($current_file_repeats == ($max_entries_output_tables + 1)){
		#
		#print the file
		#
		my $is_last = 0;
		print_files($is_last,$next_output_file_number,$txt_file_header);
		$next_output_file_number ++;
		#
		#Update indexes
		#

		my $first_repeat_index = $positions_start[0];
		my $last_repeat_index= pop @positions_end;
		push @positions, "$first_repeat_index to $last_repeat_index";
		#
		#reset variables
		#
		$current_file_repeats = 1;
		@positions_start = ();
		@positions_end = ();
	    }
	    #
	    #For HTML output, we keep track of the total number of repeats selected
	    #
	    $current_sequence_selected_repeats ++;	

	    #
	    #Add repeat start and end to statistics
	    #
	    push @positions_start, $repeat_start;
	    push @positions_end, $repeat_end;			    

	    #
	    #Add entry to HTML file output
	    #
	    
	    $html_data_string .=  "<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER><A HREF=\"$output_file_prefix\_TRAP_$original_file_prefix.$next_output_file_number.txt.html\#$html_link$repeat_start--$repeat_end<\/A><\/CENTER><\/TD>".
			    "<TD><CENTER>$period<\/CENTER><\/TD>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$copynumber<\/CENTER><\/TD>".
			    "<TD><CENTER>$consensus_size<\/CENTER><\/TD>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$match_percentage<\/CENTER><\/TD>".
			    "<TD><CENTER>$percente_indels<\/CENTER><\/TD>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$score<\/CENTER><\/TD>".
			    "<TD><CENTER>$adenine<\/CENTER><\/TD>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$citose<\/CENTER><\/TD>".
			    "<TD><CENTER>$guanine<\/CENTER><\/TD>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$timine<\/CENTER><\/TD><\/TR>\n";
	    #
	    #we need to remove eventual HTML-ending tags from the parsed entry
	    #

	    $entry =~ s/Done\.\n<\/PRE><\/BODY><\/HTML>//;
	    $txt_data_string .= $entry;


	}#End HTML output generation    
	
	#
	#generate entry in the feature table, if it is the case
	#
	if ($create_feature_table eq "yes"){
	   	    
	    $feature_table_file .= feature_table_trf ($repeat_start,$repeat_end,$score,$consensus_sequence,$trf_parameters,$consensus_size,$copynumber,$version);
	    
	}

	if ($create_feature_table_artemis eq "yes"){
	   	    
	    $feature_table_file_artemis .= feature_table_trf_artemis ($repeat_start,$repeat_end,$score,$consensus_sequence,$trf_parameters,$consensus_size,$copynumber,$version);
	    
	}


	if ($create_gff eq "yes"){
	   	    
	    $gff_table_file .= gff_table_trf ($repeat_start,$repeat_end,$score,$consensus_sequence,$trf_parameters,$consensus_size,$copynumber,$version);
	    
	}

	#
	#Update global statistics
	#
	$total_selected_repeats ++;
	$total_selected_bases += ($repeat_end - $repeat_start)+1;
	$copy_number_sum += $copynumber;
	
	#
	#do we generate summary tables?
	#
	if ($table_format){

	    #
	    #Statistics are computed for a repeat and its
	    #various "versions", generated by different reading frames
            #and reverse complementary sequences.
	    #Therefore, we ascribe a unique repeat sequence  to each such
            #set 
	    #For each code we choose one of the variations as
	    #the representative one, that will appear in the reports
	    #

	    #
	    #If the repeat has been found before, just update numbers
	    #

	    if (exists($repeat_code{$consensus_sequence})){

		my $code = $repeat_code{$consensus_sequence};
		$total_number_of_repetitive_regions[$code]++;
		$total_copies_per_repeat[$code] += $copynumber;
		$sum_of_sizes_for_repeat[$code] += ($repeat_end - $repeat_start)+1;
		$html_page_repeat_group[$code] .= "<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER><A HREF=\"$output_file_prefix\_TRAP_$original_file_prefix.$next_output_file_number.txt.html\#$html_link$repeat_start--$repeat_end<\/A><\/CENTER><\/TD>;".
		    "$period;".
		    "$copynumber;".
		    "$consensus_size;".
		    "$match_percentage;".
		    "$percente_indels;".
		    "$score;".  
		    "$adenine;".
		    "$citose;".
		    "$guanine;".
		    "$timine;".
		    "$consensus_sequence;".
		    "$sequence_name\n";
	    }

	    #
	    #otherwise, we have to compute the variations, assign the unique repeat
	    #number to them, and initialize numbers
	    #

	    else {
		my %variation = compute_variations($consensus_sequence);
		foreach my $one_variation (keys %variation){
		    $repeat_code{$one_variation} = $next_repeat_code;
		}

		$repeat_code_representative[$next_repeat_code] = $consensus_sequence;		
		$total_number_of_repetitive_regions[$next_repeat_code] = 1;
		$total_copies_per_repeat[$next_repeat_code] =  $copynumber;
		$sum_of_sizes_for_repeat[$next_repeat_code] = ($repeat_end - $repeat_start)+1;
		$size_ms[$next_repeat_code] =  length($consensus_sequence);

		$html_page_repeat_group[$next_repeat_code] .= "<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER><A HREF=\"$output_file_prefix\_TRAP_$original_file_prefix.$next_output_file_number.txt.html\#$html_link$repeat_start--$repeat_end<\/A><\/CENTER><\/TD>;".
		    "$period;".
		    "$copynumber;".
		    "$consensus_size;".
		    "$match_percentage;".
		    "$percente_indels;".
		    "$score;".  
		    "$adenine;".
		    "$citose;".
		    "$guanine;".
		    "$timine;".
		    "$consensus_sequence;".
		    "$sequence_name\n";
		$next_repeat_code ++;
	    }
	}#end table generation
	
	#
	#Output fasta data, if necessary
	#
	if ($create_consensus_file eq "yes"){
	    print CONSENSUS "\n>$sequence_name TRAP $repeat_start to $repeat_end\n";
	    print CONSENSUS "Consensus sequence ($consensus_size bp)\n$consensus_sequence\n" ;
	}
	if ($create_flanking_region_file eq "yes"){
	    print FLANK_SEQ "\n>$sequence_name TRAP $repeat_start to $repeat_end\n";
	    #
	    #pretty printing: only 70 bases per line
	    #
	    my $flaking_region_sequence = $flanking_upstream."nnnnnnnnnnnnnnnnnnnn".$flanking_downstream;
	    my $flaking_region_sequence_print = "";
	    while ($flaking_region_sequence){
		$flaking_region_sequence_print = substr ($flaking_region_sequence,0,70,"")."\n";
		print FLANK_SEQ $flaking_region_sequence_print;
	    }
	}
	
	#
	#redundancy information update
	#
	if (($sort_field) || ($create_redundancy_report eq "yes")){
	    push @repeat_region_list, "$repeat_start $repeat_end ,$period,$copynumber,$consensus_size,$match_percentage,$percente_indels,$score";
	}
    }
}
#end of @input_file foreach

#
#update redudancy information for last entry
#
if (($sort_field) || ($create_redundancy_report eq "yes")){
    $total_size_of_repeat_regions += repeat_regions_analysis (\@repeat_region_list,$create_redundancy_report,$input_file_prefix_old);
}
    
#
#reset reading delimiter
#
$/ = "\n";
#
#close files
#
close (TRFDATA);
close (FLANK_SEQ);
close (CONSENSUS);
#
#Generate HTML ouput for the last input file(s)
#
if (($create_trf_like_file eq "yes") && ($positions_start[0])){
    #
    #get last repeat's start and end
    #
    my $index_start = $positions_start[0];
    my $index_end = pop @positions_end;
    push @positions, "$index_start to $index_end";
    #
    #print stuff
    #
    my $is_last = 1;
    print_files($is_last, $next_output_file_number,$txt_file_header);

    #
    #Update summary
    #

    my $summary_file_name = "html_data\/$output_file_prefix\_TRAP_$original_file_prefix";

    if ($next_output_file_number > 1){
	print_html_index($output_directory,"$output_file_prefix\_TRAP_$original_file_prefix",\@positions);
	$summary_file_name .= "_index.html"; 
    }
    else{
	$summary_file_name  .= ".1.html";
    }
    #
    #update summary
    #
    my $summary_entry =  "<TR>".
	"<TD BGCOLOR=\"\#FFCC66\"><CENTER>".$fasta_number."<\/CENTER><\/TD>".
	"<TD><CENTER><A TARGET=\"$summary_file_name\" HREF=\"$summary_file_name\">".$sequence_name."<\/A><\/CENTER><\/TD>".
	"<TD BGCOLOR=\"\#FFCC66\"><CENTER>".$current_sequence_selected_repeats."<\/CENTER><\/TD><\/TR>\n";

    push @file_summary, $summary_entry;
}

#
#Generate feature table ouput for the last input file(s)
#

if (($create_feature_table eq "yes") && ($feature_table_file)){

    my $feature_file;   
    if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	$feature_file = $1;   
    }
    else{
	$original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	$feature_file = $1;    
    }
 
    #
    #to keep filenames small, use only fist 25 characteres of sequence name for the feature table file
    #(also, special characters are substituted for "_")
    #
    
    my $sequence_name_aux = substr($sequence_name,0,25);
    
    $sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
    $sequence_name_aux =~ s/\s/_/;
    $sequence_name_aux =~ s/\t/_/;

    $sequence_name_number_sequence_ft .= "$sequence_name;$output_file_prefix\_$feature_file\_$sequence_name_aux.tab\n";
    
    open (FEATURE_TABLE_FILE,">$output_directory/feature_table/$output_file_prefix\_$feature_file\_$sequence_name_aux.tab");
    print FEATURE_TABLE_FILE "$feature_table_file\n";
    close (FEATURE_TABLE_FILE);
    
    #
    #print a file with the sequence name and the name o feature table
    #
    open (FEATURE_TABLE_FILE_NAME_NUMBER,">$output_directory/feature_table/$output_file_prefix\_index.csv");
    print FEATURE_TABLE_FILE_NAME_NUMBER "Sequence name;Feature table file name\n";
    print FEATURE_TABLE_FILE_NAME_NUMBER "$sequence_name_number_sequence_ft";
    close (FEATURE_TABLE_FILE_NAME_NUMBER);

    $feature_table_file = "";
    
}

#
#Generate ARTEMIS feature table ouput for the last input file(s)
#

if (($create_feature_table_artemis eq "yes") && ($feature_table_file_artemis)){

    my $feature_file;   
    if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	$feature_file = $1;   
    }
    else{
	$original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	$feature_file = $1;    
    }
 
    #
    #to keep filenames small, use only fist 25 characteres of sequence name for the feature table file
    #(also, special characters are substituted for "_")
    #
    
    my $sequence_name_aux = substr($sequence_name,0,25);
    
    $sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
    $sequence_name_aux =~ s/\s/_/;
    $sequence_name_aux =~ s/\t/_/;

    $sequence_name_number_sequence_ft .= "$sequence_name;$output_file_prefix\_$feature_file\_$sequence_name_aux.tab\n";
    
    open (FEATURE_TABLE_FILE_ARTEMIS,">$output_directory/feature_table_artemis/$output_file_prefix\_$feature_file\_$sequence_name_aux.tab");
    print FEATURE_TABLE_FILE_ARTEMIS "$feature_table_file_artemis\n";
    close (FEATURE_TABLE_FILE_ARTEMIS);
    
    #
    #print a file with the sequence name and the name o feature table
    #
    open (FEATURE_TABLE_FILE_NAME_NUMBER_ARTEMIS,">$output_directory/feature_table_artemis/$output_file_prefix\_index.csv");
    print FEATURE_TABLE_FILE_NAME_NUMBER_ARTEMIS "Sequence name;Feature table file name\n";
    print FEATURE_TABLE_FILE_NAME_NUMBER_ARTEMIS "$sequence_name_number_sequence_ft";
    close (FEATURE_TABLE_FILE_NAME_NUMBER_ARTEMIS);

    $feature_table_file_artemis = "";
    
}

#
#Generate gff table ouput for the last input file(s)
#

if (($create_gff eq "yes") && ($gff_table_file)){

    my $gff_file;   
    if ($original_file_prefix =~ m/(.*\.s\d+)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/){
	$gff_file = $1;   
    }
    else{
	$original_file_prefix =~ m/(.*)\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+\.\d+/;
	$gff_file = $1;    
    }
 
    #
    #to keep filenames small, use only fist 25 characteres of sequence name for the gff file
    #(also, special characters are substituted for "_")
    #

    my $sequence_name_aux = substr($sequence_name,0,25);

    $sequence_name_aux =~ tr/ \|\`\\'\"\\\/\*\,/_/;
    $sequence_name_aux =~ s/\s/_/;
    $sequence_name_aux =~ s/\t/_/;

    $sequence_name_number_sequence_gff .= "$sequence_name;$output_file_prefix\_$gff_file\_$sequence_name_aux.gff\n";
    
    open (GFF_TABLE_FILE,">$output_directory/gff/$output_file_prefix\_$gff_file\_$sequence_name_aux.gff");
    print GFF_TABLE_FILE "$gff_table_file\n";
    close (GFF_TABLE_FILE);
    
    #
    #print a file with the sequence name and gff file name
    #
    open (GFF_TABLE_FILE_NAME_NUMBER,">$output_directory/gff/$output_file_prefix\_index.csv");
    print GFF_TABLE_FILE_NAME_NUMBER "Sequence name;GFF table file name\n";
    print GFF_TABLE_FILE_NAME_NUMBER "$sequence_name_number_sequence_gff";
    close (GFF_TABLE_FILE_NAME_NUMBER);

    $gff_table_file = "";
    
}

#
#We have gathered all statistics. Now we can print
# the summary tables, if requested
#

if ($table_format){

    my @table;
    #
    #CREATE THE TABLE
    #
    my ($table, $length_counter, $total_number_of_copies_size) = create_complete_table(\@total_copies_per_repeat,
										       $next_repeat_code,
										       \@total_number_of_repetitive_regions,
										       \@size_ms,
										       \@repeat_code_representative,
										       \@sum_of_sizes_for_repeat,
										       \%length_counter,
										       \%total_number_of_copies_size,
										       \@table);
  
    #
    #SORT THE TABLE
    #

    my %collumn;
    my @sorted_table;
    my @lines;
    my $header = shift(@$table);

    #
    #we associate to each sort field an index
    #

    $collumn{bases}   = 0;
    $collumn{loci}    = 1;
    $collumn{units}   = 2;
    $collumn{average} = 3; 
    $collumn{size}    = 4;

    #
    #sort the table according to the specified field
    #

    my $sort_field_index = $collumn{$sort_field};

    if ($sort_field =~ /size/){
	@sorted_table = sort { extract($sort_field_index,";",$a) <=> extract($sort_field_index,";",$b)} @$table;
    }
    else{
	@sorted_table = sort { extract($sort_field_index,";",$b) <=> extract($sort_field_index,";",$a)} @$table;
    }

    #
    #PRINT THE TABLE
    #
    my @summary_table = build_summary_table(\%length_counter,
					    \%total_number_of_copies_size);

   
    $copy_number_sum = int($copy_number_sum*100)/100; #we want only two decimal digits;

    if ($table_format =~ /html/i){
	print_complete_table_html(\@sorted_table,
				  $output_directory, 
				  $output_file_prefix."_TRAP_complete_table",
				  $sort_field);
	print_summary_table_html(\@summary_table,
				 $total_selected_repeats,
				 $total_selected_bases,
				 $copy_number_sum,
				 $total_size_of_repeat_regions);

	print_repeat_group_html(\@html_page_repeat_group,
				  $output_directory, 
				  $output_file_prefix."_TRAP_repeat_group",
				  $next_repeat_code);

    }
    if ($table_format =~ /csv/i){
	print_complete_table_csv(\@sorted_table, 
				 $output_directory, 
				 $output_file_prefix."_TRAP_complete_table");

	print_summary_table_csv(\@summary_table,
				$total_selected_repeats,
				$total_selected_bases,
				$copy_number_sum,
				$total_size_of_repeat_regions);
    }
}

#
#Print HTML data 
#

if ($create_trf_like_file eq "yes"){
    my $html_header = "<HTML><HEAD><TITLE>Output Summary<\/TITLE><\/HEAD>\n".
	              "<BODY bgcolor=\"\#FFFFCC\"><PRE>".
		      "TRAP (Tandem Repeats Analysis Program) written by\:\n".
		      "Tiago J.P. Sobreira\n".
		      "Alan M. Durham\n".
		      "Arthur Gruber\n".
		      "University of So Paulo\n".
		      "<B>Multiple Sequence Summary<\/B>\n".
		      "\n".
		      "Only sequences containing repeats are shown!\n".
		      "\n".
		      "Click on the sequence description to view a repeat table.\n".
		      "\n".
		      "<TABLE BORDER=3 CELLSPACING=1 CELLPADDING=1><TR>".
		      "<TD WIDTH=80 BGCOLOR=\"\#FFCC66\"><CENTER>Sequence Index<\/CENTER><\/TD>".
                      "<TD WIDTH=400><CENTER>Sequence Description<\/CENTER><\/TD>".
                      "<TD WIDTH=80 BGCOLOR=\"\#FFCC66\"><CENTER>Number of Repeats<\/CENTER><\/TD>".
                      "<\/TR>\n";

    #
    #Sort the entries
    #
    
    @file_summary = 
	sort {
	    #
	    #we need to extract the sorting info
	    #
	    $a =~ m/<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER>(\d+)/;
	    my $key_a = $1;
	    $b =~ m/<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER>(\d+)/;
	    my $key_b = $1;
	    $key_a <=> $key_b;
	} @file_summary;
    
    #
    #print the data
    #
    open (SUMMARY, ">$output_prefix"."_TRAP_summary_TRF_like.html");
    print SUMMARY  $html_header,
		  "@file_summary\n",
		  "</TABLE>\n",
		  "</BODY></HTML>\n";
    close(SUMMARY);
}

exit;

sub extract {
    #
    #Extracts a field from a table entry
    #
    my $index = shift; #index of the field to be extracted
    my $separator = shift; #field_separator
    my $the_string = shift;#table entry as a string
	
    my @stuff = split($separator,$the_string);
    return $stuff[$index];
}

sub create_complete_table{
    #
    #Generates the summary tables from the input data
    #

    my @repeats;
    my $copies_per_repeat = shift;
    my $next_repeat_code = shift;
    my $regions_per_repeat = shift;
    my $sum_of_sizes = shift;
    my $representative = shift;
    my $sum_of_sizes_for_repeat = shift;
    my $length_counter = shift;
    my $total_number_of_copies_size = shift;
    my $table = shift;

    push @$table, "Total number of bases;Total number of repeat loci;Total number of repeat units;Average number of repeat units per locus;Period size;sequence\n";

    for (my $repeat_index = 0; $repeat_index < $next_repeat_code; $repeat_index++){

	$length_counter{@$sum_of_sizes[$repeat_index]} += @$regions_per_repeat[$repeat_index] ;
	$total_number_of_copies_size{@$sum_of_sizes[$repeat_index]} += @$copies_per_repeat[$repeat_index];

	#
	#we only want two digits after the decimal point
	#
	my $average_copy_number = @$copies_per_repeat[$repeat_index]/@$regions_per_repeat[$repeat_index];

	if ($average_copy_number =~ m/(\d*\.\d\d)/){
	    $average_copy_number = $1;
	}
	
	if (@$copies_per_repeat[$repeat_index] =~ m/(\d*\.\d\d)/){
	    @$copies_per_repeat[$repeat_index] = $1;
	}
	
	push @$table, "$sum_of_sizes_for_repeat[$repeat_index];@$regions_per_repeat[$repeat_index];@$copies_per_repeat[$repeat_index];$average_copy_number;@$sum_of_sizes[$repeat_index];@$representative[$repeat_index];$repeat_index\n";

    }

    return ($table, $length_counter, $total_number_of_copies_size);
}

sub print_complete_table_csv{
    #
    #We have this function for homogeneity,
    #the table is already in CVS format
    #we only need to add the header
    #
    my $table = shift;
    my $path = shift;
    my $prefix = shift;

    open (CSV_OUTPUT_TABLE, ">$path$dir_separator$prefix.csv");

    print CSV_OUTPUT_TABLE "Total number of bases;Total number of repeat loci;",
                           "Total number of repeat units;Average number of repeat units/locus;Period size;sequence\n";
    foreach my $line (@$table){

	$line =~ m/(.*)\;\d+$/;
	$line = $1;

	print CSV_OUTPUT_TABLE "$line\n";
    }
}

sub print_complete_table_html{

    my $table = shift;
    my $path = shift;
    my $prefix = shift;
    my $sort_field = shift;

    my $first_index;
    my $last_index;
    my @positions_complete_table;    

    my $file_number = 1;

    open (HTML_OUTPUT_TABLE, ">$path/html_data/$dir_separator$prefix\_$file_number.html")
	or die "could not open $path/html_data/$dir_separator$prefix\_$file_number.html";

    my $auxiliary_javascript_functions = "";
    my $table_body = "";
    my $copy_number_print;
    my $java_script_number = 0;
    my $table_entry_number = 0;
    	       
    my $table_header = "\n\n<BODY bgcolor=\"\#FFFFCC\">\n".
	"<TABLE BORDER=3 CELLSPACING=1 CELLPADDING=1><CENTER>\n".
	"<TR><TD WIDTH=100 BGCOLOR=\"\#FFCC66\"><CENTER>Total number of bases</CENTER></TD>\n".
	"<TD WIDTH=200><CENTER>Total number of repeat <i>loci<\/i></CENTER></TD>\n".
	"<TD WIDTH=200 BGCOLOR=\"\#FFCC66\"><CENTER>Total number of repeat units</CENTER></TD>\n".
	"<TD WIDTH=200><CENTER>Average number of repeat units per <i>locus<\/i></CENTER></TD>\n".
	"<TD WIDTH=200 BGCOLOR=\"\#FFCC66\"><CENTER>Period size</CENTER></TD>\n".
	"<TD WIDTH=200><CENTER>Sequence</CENTER></TD>".
       	"</TR>\n";
	
    foreach my $table_entry (@$table){
 	$table_entry_number ++;
        #
        #We do not create a single file with all data, as there is a limit
	#on the number of entries per HTML page set in 
	#the global variable $max_entries_output_tables
	#

	if ($table_entry_number == $max_entries_output_tables){
	    #
	    #Generate previous/next links, update file number
	    #
	    my $html_link = "";
	    if ($file_number > 1) {
		$html_link .= "<a href=\"$prefix\_".($file_number-1).".html\">previous page</a>   ";
	    }
	    $html_link .= "<a href=\"$prefix\_".($file_number+1).".html\">next page</a>\n";

	    print HTML_OUTPUT_TABLE $auxiliary_javascript_functions;
	    print HTML_OUTPUT_TABLE $html_link;
	    print HTML_OUTPUT_TABLE $table_header;
	    print HTML_OUTPUT_TABLE "\n$table_body\n\n";
	    print HTML_OUTPUT_TABLE "</PRE> </TABLE></BODY></HTML>\n";

	    close(HTML_OUTPUT_TABLE);
	    $auxiliary_javascript_functions = "";
	    #
	    #Reset variables
	    #
	    $table_entry_number = 0;
	    my $auxiliary_javscript_functions = "";
	    $table_body = "";
	    
	    #
	    #Open the next file
	    #
	    $file_number++;
	    open (HTML_OUTPUT_TABLE, ">$path/html_data/$dir_separator$prefix\_$file_number.html");
	    push  @positions_complete_table , "$first_index to $last_index";
            $first_index = "";
	}           

	#
	# Now we can process the new entry
	#
	my $sequence_html;
	
	my ($bases,
	    $repeat_loci,
	    $repeat_units,
	    $average_rep_units_loci,
	    $size,
	    $sequence,
	    $link)
	               = split (/;/,$table_entry);
	chomp $sequence;

        #
        #complete table abbreviations
        #

	my %field;
	$field{bases}   = $bases;
	$field{loci}    = $repeat_loci;
	$field{units}   = $repeat_units;
	$field{average} = $average_rep_units_loci; 
	$field{size}    = $size;
	#
	#for each page, we keep track of first and last
	#index. 
	#
	
	#
	#if first index has no value, then we update both
	#
	
	if (!$first_index) {
	    $first_index = $field{$sort_field};
	}
	#
	#last index is always updated
	#

        $last_index = $field{$sort_field};

	#
	#if a sequence is longer than 20 bp, we insert a pop-up window for visualizing
	#the sequence (looks better this way)
	#
	if ($size > 20){

	    $java_script_number++;
	    $sequence_html = "<FORM name=\"formulario\"><INPUT TYPE=\"BUTTON\" VALUE=\"Sequence\" onClick=\"Rep$java_script_number();\"></FORM>";

	    my $sequence_format;
	    if (length $sequence > 50){
		while (my $next_50 = substr($sequence,0,50)){

		    $sequence_format .= $next_50."<BR>";
		    $sequence = substr($sequence,50);

		    if (length $sequence < 50){
			$sequence_format .= $sequence;
			last;
		    }
		}
		$sequence = $sequence_format;
	    }

	    $auxiliary_javascript_functions .= "\n".generate_javascript($java_script_number, $sequence,$link);

	}
	else {
	    $sequence_html = $sequence;
	}
			    
	$table_body .= 
	    "<TR>\n".
            "<TD WIDTH=100 BGCOLOR=\"\#FFCC66\"><CENTER>$bases</CENTER></TD>\n".
	    "<TD WIDTH=200><CENTER>$repeat_loci</CENTER></TD>\n".
	    "<TD WIDTH=200 BGCOLOR=\"\#FFCC66\"><CENTER>$repeat_units</CENTER></TD>\n".
	    "<TD WIDTH=200><CENTER>$average_rep_units_loci</CENTER></TD>\n".
	    "<TD WIDTH=200 BGCOLOR=\"\#FFCC66\"><CENTER>$size</CENTER></TD>\n".
"<TD WIDTH=200><CENTER><a target=\"$output_file_prefix\_TRAP_repeat_group_$link\_1.html\"  href=\"$output_file_prefix\_TRAP_repeat_group_$link\_1.html\">$sequence_html</CENTER></TD>\n".
	    "</TR>\n\n";
    }

    #
    #Print index file of Complete Table HTML
    #

    if ($first_index){
        push (@positions_complete_table , "$first_index to $last_index");
        print_html_index_complete_table ($output_directory,"$prefix",\@positions_complete_table);
    }
    #
    # The last table needs to be printed
    #

    my $html_link = "";
    if ($file_number > 1) {
	$html_link .= "<a href=\"$prefix\_".($file_number-1).".html\">previous page</a><br>";
    }
	
    print HTML_OUTPUT_TABLE $auxiliary_javascript_functions;
    print HTML_OUTPUT_TABLE $html_link;

    print HTML_OUTPUT_TABLE $table_header;
    print HTML_OUTPUT_TABLE  "\n$table_body\n\n";
    print HTML_OUTPUT_TABLE "</TABLE></BODY></HTML>\n";
    close(HTML_OUTPUT_TABLE);

}


#################
################
#############
##########
###
##tiago
sub print_repeat_group_html{

    my $html_page_repeat_group = shift;
    my $path = shift;
    my $prefix = shift;
    my $next_repeat_code = shift;
    my $file_group = 0; 
    my $table_entry;
    my $table_header = "<TABLE BORDER=1 CELLSPACING=0 CELLPADDING=0>".
		       "<TR><TD WIDTH=140 BGCOLOR=\"\#FFCC66\"><CENTER>Indices</CENTER></TD>".
		       "<TD WIDTH=80><CENTER>Period<BR>Size </CENTER></TD>".
		       "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Copy<BR>Number</CENTER></TD>".
		       "<TD WIDTH=70><CENTER>Consensus<BR>Size</CENTER></TD>".
		       "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Percent<BR>Matches</CENTER></TD>".
		       "<TD WIDTH=70><CENTER>Percent<BR>Indels</CENTER></TD>".
		       "<TD WIDTH=60 BGCOLOR=\"\#FFCC66\"><CENTER>Score</CENTER></TD>".
		       "<TD WIDTH=40><CENTER>A</CENTER></TD>".
		       "<TD WIDTH=40 BGCOLOR=\"\#FFCC66\"><CENTER>C</CENTER></TD>".
		       "<TD WIDTH=40><CENTER>G</CENTER></TD>".
		       "<TD WIDTH=40 BGCOLOR=\"\#FFCC66\"><CENTER>T</CENTER></TD>".
		       "<TD WIDTH=40><CENTER>Consensus_sequence</CENTER></TD>".
                       "<TD WIDTH=80 BGCOLOR=\"\#FFCC66\"><CENTER>Prefix Sequence Name</CENTER></TD></TR>\n";

    for ($file_group = 0; $file_group < $next_repeat_code; $file_group ++){

	if (@$html_page_repeat_group[$file_group]){
	    $table_entry = @$html_page_repeat_group[$file_group];
	}
	else{ 
	    next;
	}

	my $file_number = 1;  
	open (HTML_OUTPUT_TABLE, ">$path/html_data/$dir_separator$prefix\_$file_group\_$file_number.html")
	    or die "could not open $path/html_data/$dir_separator$prefix\_$file_group\_$file_number.html";
	
	my $auxiliary_javascript_functions = "";
	my $table_body = "";
	my $copy_number_print;
	my $java_script_number = 0;
	my $table_entry_number = 0;
	
	my @repeat_group = split (/\n/,$table_entry);


	 @repeat_group =  sort {my @a1 = (split (/;/, $a));
				  my @b1 = (split (/;/, $b));    
				  
				  $a1[0] =~ m/(\d+)--\d+/;
				  my $start_position_a = $1;
				  
				  $b1[0] =~ m/(\d+)--\d+/;
				  my $start_position_b = $1;

				  $a1[12] cmp $b1[12]
				      ||
				  $start_position_a <=>  $start_position_b;
			      }@repeat_group;


	foreach my $table_entry (@repeat_group){
	    $table_entry_number++;

	    #
	    #We do not create a single file with all data, as there is a limit
	    #on the number of entries per HTML page set in 
	    #the global variable $max_entries_output_tables
	    #

	    if ($table_entry_number == $max_entries_output_tables){
		#
		#Generate previous/next links, update file number
		#
		my $html_link = "";
		if ($file_number > 1) {
		    $html_link .= "<a href=\"$prefix\_$file_group\_".($file_number-1).".html\">previous page</a>   ";
		}
		$html_link .= "<a href=\"$prefix\_$file_group\_".($file_number+1).".html\">next page</a>\n";

		print HTML_OUTPUT_TABLE $auxiliary_javascript_functions;
		print HTML_OUTPUT_TABLE $html_link;
		print HTML_OUTPUT_TABLE $table_header;
		print HTML_OUTPUT_TABLE "\n$table_body\n\n";
		print HTML_OUTPUT_TABLE "</PRE> </TABLE></BODY></HTML>\n";

		close(HTML_OUTPUT_TABLE);
		$auxiliary_javascript_functions = "";
		#
		#Reset variables
		#
		$table_entry_number = 0;
		my $auxiliary_javscript_functions = "";
		$table_body = "";
		
		#
		#Open the next file
		#
		$file_number++;
		open (HTML_OUTPUT_TABLE, ">$path/html_data/$dir_separator$prefix\_$file_group\_$file_number.html")
		    or die "could not open $path/html_data/$dir_separator$prefix\_$file_group\_$file_number.html";
	    }           

	    #
	    # Now we can process the new entry
	    #
	    my $sequence_html;
	    my ($indices,
		$period,
		$copy_number,
		$size,
		$percent_matches,
		$percent,
		$score,
		$A,
		$C,
		$G,
		$T,
		$sequence,
		$sequence_name)
		= split (/;/,$table_entry);
	    chomp $sequence;

	    if (length $sequence_name > 20){
		$sequence_name = substr($sequence_name,0,20);
	    }
	    #
	    #if a sequence is longer than 20 bp, we insert a pop-up window for visualizing
	    #the sequence (looks better this way)
	    #
	    if ($size > 20){

		$java_script_number++;
		$sequence_html = "<FORM name=\"formulario\"><INPUT TYPE=\"BUTTON\" VALUE=\"Sequence\" onClick=\"Rep$java_script_number();\"></FORM>";

		my $sequence_format;
		if (length $sequence > 50){
		    while (my $next_50 = substr($sequence,0,50)){

			$sequence_format .= $next_50."<BR>";
			$sequence = substr($sequence,50);

			if (length $sequence < 50){
			    $sequence_format .= $sequence;
			    last;
			}
		    }
		    $sequence = $sequence_format;
		}
		$auxiliary_javascript_functions .= "\n".generate_javascript($java_script_number, $sequence);
	    }
	    else {
		$sequence_html = $sequence;
	    }
	    


	    


	    $table_body .= "$indices\n".
			   "<TD><CENTER>$period<\/CENTER><\/TD>".
			   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$copy_number<\/CENTER><\/TD>".
			   "<TD><CENTER>$size<\/CENTER><\/TD>".
			   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$percent_matches<\/CENTER><\/TD>".
			   "<TD><CENTER>$percent<\/CENTER><\/TD>".
			   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$score<\/CENTER><\/TD>".
			   "<TD><CENTER>$A<\/CENTER><\/TD>".
			   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$C<\/CENTER><\/TD>".
			   "<TD><CENTER>$G<\/CENTER><\/TD>".
			   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$T<\/CENTER><\/TD>".
			   "<TD><CENTER>$sequence_html</CENTER></TD>".
	                   "<TD BGCOLOR=\"\#FFCC66\"><CENTER>$sequence_name </CENTER></TD></TR>\n";
	}

	#
	# The last table needs to be printed
	#

	my $html_link = "";
	if ($file_number > 1) {
	    $html_link .= "<a href=\"$prefix\_$file_group\_".($file_number-1).".html\">previous page</a><br>";
	}
	
	print HTML_OUTPUT_TABLE "$auxiliary_javascript_functions\n";
	print HTML_OUTPUT_TABLE "$html_link\n";

	print HTML_OUTPUT_TABLE "$table_header\n";
	print HTML_OUTPUT_TABLE "\n$table_body\n\n";
	print HTML_OUTPUT_TABLE "</TABLE></BODY></HTML>\n";
	close(HTML_OUTPUT_TABLE);

    }
}
#######
###########
#################
###################
  

sub build_summary_table {
    #
    #Receives two hashes, 
    #   %size_to_number_of_repeats, associating a repeat size to the total number
    #           of repeats of that given size
    #   %length_counter, associating to a period size, the number of loci 
    #           with periods of that size
    #Returns that associates to each period size
    #              the number of loci with that period size
    #              the number of repeat copies with that period size
    #              the ratio of the two previous one, indicating the average
    #                      number of repeats per locus for that period size
    #

    my $length_counter = shift;
    my $total_number_of_copies_size = shift;
    my @summary_table;
    
    #
    #Sort according to the length of the repetetive unit
    #

    foreach my $length (sort{$a <=> $b}  (keys %$length_counter)){
	
	my $ratio = $total_number_of_copies_size{$length}/$length_counter{$length};
	my $total_copies = $total_number_of_copies_size{$length};
	
        #
	#We only want the two most significant digits
	#
	$ratio = int($ratio*100)/100;
	$total_copies = int($total_copies*100)/100;

	#
	#next entry of summary table
	#
	
	push @summary_table, "$length;$length_counter{$length};$total_copies;$ratio\n";
	
    }
    return @summary_table;
}

sub print_summary_table_csv{
    
    #
    #No conversion necessary, we just add the header and print
    #one line at a time
    #
    
    my $summary_table = shift;
    my $total_repeats = shift;
    my $total_bases = shift;
    my $total_copies = shift;
    my $total_size_of_repeat_regions = shift;
    open (TAB_CSV, ">$output_prefix"."_TRAP_summary_table.csv");

    #
    #header
    #

    print TAB_CSV "Period size;",
                  "Total number of repeat loci;",
                  "Total number of repeat units;",
                  "Average number of repeat units/locus\n";

    #
    #table body
    #
    foreach my $summary_input (@$summary_table){
	print TAB_CSV $summary_input;
    }
    
    #
    #if there is no regions with repeats the Total number of repeat bases excluding base redundancy 
    #is equal to Total number of repeat bases
    #

    if ($total_size_of_repeat_regions < 1){
	$total_size_of_repeat_regions = $total_selected_bases;
    }

    #
    #table summary
    #
    print TAB_CSV "Total number of  repeats : $total_repeats\n",
                  "Total number of repeat bases : $total_selected_bases\n",
                  "Total number of repeat copies : $total_copies\n\n",
                  "Total number of repeat bases excluding base redundancy : $total_size_of_repeat_regions\n";
}

sub print_summary_table_html{
    
    my $summary_table = shift;
    my $total_repeats = shift;
    my $total_bases = shift;
    my $total_copies = shift;
    my $total_size_of_repeat_regions = shift;

    open (TAB_HTML, ">$output_prefix"."_TRAP_summary_table.html") or die "Can not open $output_prefix"."summary_table.html";

    #
    # html header
    #
    print TAB_HTML  "<BODY bgcolor=\"\#FFFFCC\">\n",
		    "<TABLE BORDER=3 CELLSPACING=1 CELLPADDING=1><CENTER>\n",
		    "<TR><TD WIDTH=100  BGCOLOR=\"\#FFCC66\"><CENTER>Period size</CENTER></TD>\n",
	            "<TD WIDTH=200><CENTER>Total number of repeat <i>loci<\/i></CENTER></TD>\n",
		    "<TD WIDTH=200  BGCOLOR=\"\#FFCC66\"><CENTER>Total number of repeat units</CENTER></TD>\n",
		    "<TD WIDTH=200><CENTER>Average number of repeat units/<i>locus<\/i></CENTER></TD></TR>\n";

    #
    #table body
    #
    foreach my $summary_input (@$summary_table){

	#
	#separate the fields
	#
	my @input = split (/;/, $summary_input);
	
	#
	#print html table entry
	#
	print TAB_HTML  "<TR>",
			"<TD BGCOLOR=\"\#FFCC66\" ><CENTER>$input[0]</CENTER></TD>",
			"<TD><CENTER>$input[1]</CENTER></TD>",
			"<TD BGCOLOR=\"\#FFCC66\"><CENTER>$input[2]</CENTER></TD>",
			"<TD><CENTER>$input[3]</CENTER></TD>",
			"</TR>";

    }

    #
    #if there is no regions with repeats the Total number of repeat bases excluding base redundancy 
    #is equal to Total number of repeat bases
    #

    if ($total_size_of_repeat_regions < 1){
	$total_size_of_repeat_regions = $total_selected_bases;
    }


    #
    #table summary
    #
    print TAB_HTML  "</TABLE></CENTER>\n",
			 "Total number of repeat <i>loci<\/i>: $total_selected_repeats<BR>\n",
			 "Total number of repeat bases: $total_selected_bases<BR>\n",
			 "Total number of repeat copies: $total_copies\n",
                         "<BR>Total number of repeat bases excluding base redundancy : $total_size_of_repeat_regions\n",
			 "</BODY></HTML>\n";
}

sub compute_variations {

    #
    #For a given repeat,
    #computes all variations that result in a similar
    #repetitive region, considering both DNA strands
    #(for example if repeat is "ATG", returns ["ATG","TGA","GAT",
    #"TAC","ACT","CTA"])
    #Result is a hash to facilitate membership queries
    #

    my $define_repeat_sequence = shift;
    my %repeat_variations;

    #
    #compute reverse_complement
    #
    my $reversed_repeat = reverse $define_repeat_sequence;
    $reversed_repeat =~ tr/atgc/tacg/;
    $reversed_repeat =~ tr/ATGC/TACG/;
    

    #
    #To compute variations we only need to keep
    #taking out first base and adding to the end of the
    #sequence, saving each time
    #

    my $size  = length($define_repeat_sequence);
    my @repeat_array = split (//,$define_repeat_sequence);
    my @rev_repeat_array = split (//,$reversed_repeat);

    for (my $cont = 1; $cont <= $size; $cont++){
	#
	#rotate simbols in repeat
	#
	push (@repeat_array,shift(@repeat_array));
	$repeat_variations{join("",@repeat_array)} = 1;
	#
	#rotate simbols in reversed repeat
	#
	push(@rev_repeat_array,shift(@rev_repeat_array));
	$repeat_variations{join("",@rev_repeat_array)} = 1;
    }
    return %repeat_variations;
}

sub print_files {

    #
    #this file prints the html files
    #

    my $is_last_table = shift;
    my $file_number = shift;
    my $txt_file_header = shift;
    my $header =  "<TABLE BORDER=1 CELLSPACING=0 CELLPADDING=0>".
		  "<TR><TD WIDTH=140 BGCOLOR=\"\#FFCC66\"><CENTER>Indices</CENTER></TD>".
		  "<TD WIDTH=80><CENTER>Period<BR>Size </CENTER></TD>".
		  "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Copy<BR>Number</CENTER></TD>".
		  "<TD WIDTH=70><CENTER>Consensus<BR>Size</CENTER></TD>".
		  "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Percent<BR>Matches</CENTER></TD>".
		  "<TD WIDTH=70><CENTER>Percent<BR>Indels</CENTER></TD>".
		  "<TD WIDTH=60 BGCOLOR=\"\#FFCC66\"><CENTER>Score</CENTER></TD>".
		  "<TD WIDTH=40><CENTER>A</CENTER></TD>".
		  "<TD WIDTH=40 BGCOLOR=\"\#FFCC66\"><CENTER>C</CENTER></TD>".
		  "<TD WIDTH=40><CENTER>G</CENTER></TD>".
		  "<TD WIDTH=40 BGCOLOR=\"\#FFCC66\"><CENTER>T</CENTER></TD></TR>";

    my $tail = "</TABLE></PRE><P>";


    my $links = "";
    #
    #did we have a previous file for the same sequence?
    #


    if ($file_number > 1){
	#
	#If not the first file, must have link to a previous one
	#
	$links ="<A HREF=\"$output_file_prefix\_TRAP_$original_file_prefix.".
	         ($file_number-1).
		 ".html\" target=\"_self\">Previous Page<\/A>\n";
    }

    if (!$is_last_table){
	#
	#If this is not the last file, need to add link to
	#next
	#
	$links .= "<A HREF=\"$output_file_prefix\_TRAP_$original_file_prefix.".
	          ($file_number+1).
		  ".html\" target=\"_self\">Next Page<\/A>\n";
    }

    open (HTML, ">$output_directory/html_data/"."$output_file_prefix\_TRAP_$original_file_prefix.$file_number.html")
	or die "cannot open $output_file_prefix\_TRAP_$original_file_prefix$file_number.html";

    open (TXT, ">$output_directory/html_data/"."$output_file_prefix\_TRAP_$original_file_prefix.$file_number.txt.html")
	or die "cannot open $output_file_prefix\_TRAP_$original_file_prefix$file_number.txt.html";



    print HTML ("<HTML>\n<BODY bgcolor=\"\#FFFFCC\"><BASE TARGET=\"$output_file_prefix\_TRAP_$original_file_prefix.$file_number.txt.html\">",
		$links,$header,$html_data_string,$tail,$links,
		"</BODY></HTML>");

    print TXT $txt_file_header.$txt_data_string;

    close(HTML);
    close(TXT);

    #
    #some variables need to be reinitialized;
    #

    $html_data_string = "";
    $txt_data_string = "";
}

sub print_html_index {
    my $directory = shift;
    my $file_prefix = shift;
    my $all_repeat_positions = shift;

    open (HTML_INDEX_OUT_LINKS, ">$output_directory/html_data/$file_prefix"."_index_links.html") 
	or die "can not open $directory/"."$file_prefix"."_index_links.html";

    open (HTML_INDEX_OUT, ">$output_directory/html_data/$file_prefix"."_index.html") 
	or die "can not open $directory/"."$file_prefix"."_index.html";

    my $file_number_index = 1;
    
    print HTML_INDEX_OUT  "<HTML>\n".
	"<FRAMESET COLS=\"20\%, 90\%\">\n".
	"<FRAME SRC=\"$file_prefix"."_index_links.html\">\n".
	"<FRAME SRC=\"$file_prefix.$file_number_index.html\" name=\"main\">\n". 
	"</FRAMESET>\n".
	"</HTML>";

    close HTML_INDEX_OUT;
    
    print  HTML_INDEX_OUT_LINKS "<HTML><BODY bgcolor=\"\#FFFFCC\"><font color=\"\#000000\"><b>Positions of the repeat <i>loci<\/i><\/b><\/font><BR>\n";
    #
    #Add link to each output file, indexed by repeats
    #
    foreach my $aux (@$all_repeat_positions){
	print HTML_INDEX_OUT_LINKS "<A HREF=\"$file_prefix.$file_number_index.html\" target=\"main\">[$aux]<\/A><BR>\n";
	$file_number_index++;
	}
    print  HTML_INDEX_OUT_LINKS "<\/HTML>\n";
    
    close HTML_INDEX_OUT_LINKS;

}

sub print_html_index_complete_table {
    my $directory = shift;
    my $file_prefix = shift;
    my $all_repeat_positions = shift;

   open (HTML_INDEX_OUT, ">$directory/"."$file_prefix"."_index.html") 
	or die "can not open $directory/"."$file_prefix"."_index.html";

    open (HTML_INDEX_OUT_LINKS, ">$directory/html_data/"."$file_prefix"."_index_links.html") 
	or die "can not open $directory/"."$file_prefix"."_index_links.html";
    
       

    my $file_number_index = 1;
    
    print HTML_INDEX_OUT  "<HTML>\n".
	"<FRAMESET COLS=\"20\%, 90\%\">\n".
	"<FRAME SRC=\"html_data\/$file_prefix"."_index_links.html\">\n".
	"<FRAME SRC=\"html_data\/$file_prefix\_1.html\" name=\"main\">\n". 
	"</FRAMESET>\n".
	"</HTML>";

    close HTML_INDEX_OUT;
    my %sorted_field;

    $sorted_field{bases} = "total number of bases <BR> (decreasing order)";
    $sorted_field{loci} = "total number of repeat <i>loci<\/i> <BR> (decreasing order)";
    $sorted_field{units} = "total number of repeat units <BR> (decreasing order)";
    $sorted_field{average} = "average number of repeat units per <i>locus<\/i> <BR> (decreasing order)";
    $sorted_field{size} = "period size of the repeats <BR> (increasing order)";


    print  HTML_INDEX_OUT_LINKS "<HTML><BODY bgcolor=\"\#FFFFCC\"><font color=\"\#000000\"><b>Sorted by the $sorted_field{$sort_field}<\/font><BR>\n";
                                                    
    #
    #Add link to each output file, indexed by repeats
    #
    foreach my $aux (@$all_repeat_positions){
	print HTML_INDEX_OUT_LINKS "<A HREF=\"$file_prefix\_$file_number_index.html\" target=\"main\">[$aux]<\/A><BR>\n";
	$file_number_index++;
	}
    print  HTML_INDEX_OUT_LINKS "<\/b><\/BODY><\/HTML>\n";
    
    close HTML_INDEX_OUT_LINKS;

}


sub generate_javascript{
    #
    #Auxiliary function, makes code above more readable
    #generates a javascript routine that opens a window with a sequence description
    #
    my $java_script_number = shift;
    my $sequence = shift;
    my $link = shift;
    
    if ($link){
	chomp $link;
	
	return "<script language=\"JavaScript\">\n".
	       "  function Rep$java_script_number(){ \n".
	       "  var repeatwin$java_script_number = window.open(\"\",\n".
	       "			     \"jan$java_script_number\",\n".
	       "			     \"height=300,width=600,scrollbars=YES,resizable=yes\");\n".
	       "  repeatwin$java_script_number.document.write(\'<html>\\n<head>\\n<title></title>\\n</head>\\n<body><BR>\\n<b>Sequence:<BR><a target=\"$output_file_prefix\_TRAP_repeat_group_$link\_1.html\" href=\"$output_file_prefix\_TRAP_repeat_group_$link\_1.html\">$sequence<\/a><BR>\\n\');\n".
	       "  repeatwin$java_script_number.document.write(\"<FORM>\\n\");\n".
	       "  repeatwin$java_script_number.document.write(\'<INPUT TYPE=\"BUTTON\" VALUE=\"Close\" onClick=\"window.close();\">\');\n".
	       "  repeatwin$java_script_number.document.write(\"</FORM>\\n</body>\\n</html>\\n\");\n".
	       "}\n".
	       "</script>\n";
    }
    else {
	return "<script language=\"JavaScript\">\n".
	       "  function Rep$java_script_number(){ \n".
	       "  var repeatwin$java_script_number = window.open(\"\",\n".
	       "			     \"jan$java_script_number\",\n".
	       "			     \"height=300,width=600,scrollbars=YES,resizable=yes\");\n".
	       "  repeatwin$java_script_number.document.write(\'<html>\\n<head>\\n<title></title>\\n</head>\\n<body><BR>\\n<b>Sequence:<BR>$sequence<BR>\\n\');\n".
	       "  repeatwin$java_script_number.document.write(\"<FORM>\\n\");\n".
	       "  repeatwin$java_script_number.document.write(\'<INPUT TYPE=\"BUTTON\" VALUE=\"Close\" onClick=\"window.close();\">\');\n".
	       "  repeatwin$java_script_number.document.write(\"</FORM>\\n</body>\\n</html>\\n\");\n".
	       "}\n".
	       "</script>\n";
	   }
}


#
#generates "feature table" information
#
sub feature_table_trf {

  my($start_position,$end_position,$score,$seq,$parameter_list,$cons_sequence,$copy_number,$version) = @_;
  my $result; 
  my $rpt_unit_range = $start_position+$cons_sequence-1;

  $result .=  "FT   repeat_region   $start_position\..$end_position\n";
  $result .=  "FT                   /inference=\"ab initio prediction:Tandem Repeats Finder:$version\"\n";
  $result .=  "FT                   /note=\"TRF parameters $parameter_list\"\n";
  $result .=  "FT                   /note=\"TRF score is $score\"\n";
  $result .=  "FT                   /rpt_type=TANDEM\n";
  if ($cons_sequence <= 10){
      $result .=  "FT                   /rpt_unit_seq=\"$seq($copy_number)\"\n";
  }
  else{
      $result .=  "FT                   /note=\"number of repeat unit copies is $copy_number\"\n";
      $result .=  "FT                   /note=\"repeat unit size is $cons_sequence\"\n";
      $result .=  "FT                   /rpt_unit_range=$start_position\..$rpt_unit_range\n";
  }

  return $result;
}

sub feature_table_trf_artemis {

  my($start_position,$end_position,$score,$seq,$parameter_list,$cons_sequence,$copy_number,$version) = @_;
  my $result; 

  my $rpt_unit_range = $start_position+$cons_sequence-1;


  $result .=  "FT   repeat_region   $start_position\..$end_position\n";
  $result .=  "FT                   /label=repeat_region\n";
  $result .=  "FT                   /inference=\"ab initio prediction:Tandem Repeats Finder:$version\"\n";
  $result .=  "FT                   /note=\"TRF parameters $parameter_list\"\n";
  $result .=  "FT                   /note=\"TRF score is $score\"\n";
  $result .=  "FT                   /rpt_type=TANDEM\n";
  $result .=  "FT                   /color=8\n";

  if ($cons_sequence <= 10){
      $result .=  "FT                   /rpt_unit_seq=\"$seq($copy_number)\"\n";
  }
  else{
      $result .=  "FT                   /note=\"number of repeat unit copies is $copy_number\"\n";
      $result .=  "FT                   /note=\"repeat unit size is $cons_sequence\"\n";
      $result .=  "FT                   /rpt_unit_range=$start_position\..$rpt_unit_range\n";
  }
  return $result;

}


#
#generates gff information
#
sub gff_table_trf {

  my($start_position,$end_position,$score,$seq,$parameter_list,$cons_sequence,$copy_number,$version) = @_;
  my $result; 


  $result = ".	TRF	satellite	$start_position	$end_position	$score	+	.	note \"satellite sequence\" \"TRF parameters  $parameter_list\" \"repeat unit size = $cons_sequence\" \"copy number = $copy_number\" \"predicted by Tandem Repeats Finder $version\" ; label \"satellite\" ; rpt_type \"tandem\" ; rpt_unit \"$seq\" ; color 9\n";

  return $result;
}



#
#repeat_regions_analysis
#
sub repeat_regions_analysis{
    my $repeat_region_list = shift; 
    my $create_redundancy_report = shift;

    my $input_file_prefix_old = shift;

    my $repeat_data;
    my $redundante_repeat_data;
    my $print_only_once_prefix_file_name = 0;
    
    my @inicio;
    my @fim;
    my @dados;
    my @macro_regions;

    my $total_size_of_repeat_regions = 0;
    #
    #if there are no repeats, return zero
    #
    if (@$repeat_region_list < 1){
	return 0;
    }

    #
    #sorting by first position of the repeat
    #
    @repeat_region_list = 
	sort {
	    my @c = split(/\s/,$a);
	    my @d = split(/\s/,$b);      
	    $c[0] <=> $d[0];
	} @$repeat_region_list;
    #
    #To reduce further processing we will generate 2 arrays from the
    #repeat descriptions, @region_start, @region_end  and @region_data
    #
    my @region_start;
    my @region_end;
    my @region_data;

    foreach my $region (@repeat_region_list){
	(my $start, my $end, my $data) = split(/\s/,$region);
	push(@region_start, $start);
	push(@region_end, $end);
	push(@region_data, $data);
    }
    
    
    #
    #now look for each repeat region, in ascending order of start
    #position. Join repeat regions that intersect creating "macro regions"
    #these macro-regions are the true repeat regions
    #

    my $start_macro_region = $region_start[0];
    my $end_macro_region =  $region_end[0];
    my $macro_region_sub_region_list = "0";
    my $size = @repeat_region_list;
    
    if ($size > 1){
	for(my $i = 1 ; $i < $size; $i++){
	    
	    my $start = $region_start[$i];
	    my $end = $region_end[$i];
	    
	    if ($end_macro_region < $start){
		
		$total_size_of_repeat_regions += ($end_macro_region - $start_macro_region) + 1;
		push(@macro_regions, "$start_macro_region;$end_macro_region\#$macro_region_sub_region_list");
		$start_macro_region = $start;
		$end_macro_region = $end;
		$macro_region_sub_region_list = "$i";
	    }
	    else{
		$macro_region_sub_region_list .= ",$i";
		if ($end > $end_macro_region){
		    $end_macro_region = $end;
		}
	    }
	}
	
	#
	#now the last one
	#
	
	$total_size_of_repeat_regions += ($end_macro_region - $start_macro_region) + 1;
	push(@macro_regions, "$start_macro_region;$end_macro_region\#$macro_region_sub_region_list");
    }
    
    if ($create_redundancy_report eq "yes"){
	
	my @redundant_regions_link;
	
	#
	#print HTML files
	#
	
	
	
	my $header_redundant_data_file = "<TABLE BORDER=1 CELLSPACING=0 CELLPADDING=0>".
	                                 "<TR><TD WIDTH=140 BGCOLOR=\"\#FFCC66\"><CENTER>Coordinates</CENTER></TD>".
					 "<TD WIDTH=80><CENTER>Period<BR>Size </CENTER></TD>".
					 "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Copy<BR>Number</CENTER></TD>".
					 "<TD WIDTH=70><CENTER>Consensus<BR>Size</CENTER></TD>".
					 "<TD WIDTH=70 BGCOLOR=\"\#FFCC66\"><CENTER>Percent<BR>Matches</CENTER></TD>".
					 "<TD WIDTH=70><CENTER>Percent<BR>Indels</CENTER></TD>".
					 "<TD WIDTH=60 BGCOLOR=\"\#FFCC66\"><CENTER>Score</CENTER></TD>";


	my $open_repeat_redundant_file_control = 0;
	my $summary_repeat_redundancy_control = 0;
	
	foreach my $macro (@macro_regions) {

	    (my $first_part, my $second_part) = split("#",$macro);
	    (my $inicio_macro,my $fim_macro) = split(";",$first_part);
	    
	    my @sub_regions = split(",",$second_part);
	    
	    if (@sub_regions > 1){
		
		if ($open_repeat_redundant_file_control == 0){
		    open (REDUNDANT_REPEAT_REGIONS_DATA, ">$output_directory\/html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions.html") 
			or die "could not open $output_directory\/html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions.html";
		    
		    print REDUNDANT_REPEAT_REGIONS_DATA "<BODY bgcolor=\"\#FFFFCC\">\n";
		    $open_repeat_redundant_file_control = 1;
		    
		}
		print REDUNDANT_REPEAT_REGIONS_DATA "$header_redundant_data_file\n";
		foreach my $sub_region (@sub_regions){
		    my @redundant_data_aux = split (/,/, $region_data[$sub_region]);
		    
		    

		    print REDUNDANT_REPEAT_REGIONS_DATA 
			"<TR><TD BGCOLOR=\"\#FFCC66\"><CENTER>$region_start[$sub_region] to $region_end[$sub_region]<\/CENTER><\/TD>".
			"<TD><CENTER>$redundant_data_aux[1]<\/CENTER><\/TD>".
			"<TD BGCOLOR=\"\#FFCC66\"><CENTER>$redundant_data_aux[2]<\/CENTER><\/TD>".
			"<TD><CENTER>$redundant_data_aux[3]<\/CENTER><\/TD>".
			"<TD BGCOLOR=\"\#FFCC66\"><CENTER>$redundant_data_aux[4]<\/CENTER><\/TD>".
			"<TD><CENTER>$redundant_data_aux[5]<\/CENTER><\/TD>".
			"<TD BGCOLOR=\"\#FFCC66\"><CENTER>$redundant_data_aux[6]<\/CENTER><\/TD>";
		}
		print REDUNDANT_REPEAT_REGIONS_DATA 
		    "<A NAME=\"$inicio_macro--$fim_macro\"><BR><b>Redundant region: $inicio_macro to $fim_macro<\/b> <BR>\n";
		
		push (@redundant_regions_link, 
		      "<A HREF=\"$output_file_prefix\_$input_file_prefix_old\_redundant_regions.html\#$inicio_macro--$fim_macro\"target=\"main\">$inicio_macro to $fim_macro</A><BR>\n");		
		
		$summary_repeat_redundancy_control = 1;
	    }
	}
	
	close (REDUNDANT_REPEAT_REGIONS_DATA); 

	if ($summary_repeat_redundancy_control == 1){
	    my $summary_entry = "<TR>".
			    "<TD BGCOLOR=\"\#FFCC66\"><CENTER>".$fasta_number."<\/CENTER><\/TD>".
			    "<TD><CENTER><A TARGET=\"html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions_index.html\" ".
			    "HREF=\"html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions_index.html\">".$sequence_name.
			    "<\/A><\/CENTER><\/TD>".
			    "<\/TR>\n";
	    
	    push @redundant_repeat_summary_file, $summary_entry;
	}
	
	#
	#make index files
	#
	if (@redundant_regions_link){
	    
	    open (REDUNDANT_REGIONS_LINKS, ">$output_directory\/html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions_links.html") 
		or die "could not open html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions.html file" ;	
	    print REDUNDANT_REGIONS_LINKS "<HTML><BODY bgcolor=\"\#FFFFCC\"><font color=\"\#000000\"><b>Positions of the repeat <i>loci</i><BR></font><BR>\n".
		"@redundant_regions_link";
	    @redundant_regions_link = ();
	    
	    open (REDUNDANT_REGIONS_INDEX, ">$output_directory\/html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions_index.html") 
		or die "can not open $output_directory\/html_data\/$output_file_prefix\_$input_file_prefix_old\_redundant_regions_index.html";
	    
	    print REDUNDANT_REGIONS_INDEX "<HTML>\n".
		"<FRAMESET COLS=\"30\%, 90\%\">\n".
		"<FRAME SRC=\"$output_file_prefix\_$input_file_prefix_old\_redundant_regions_links.html\">\n".
		"<FRAME SRC=\"$output_file_prefix\_$input_file_prefix_old\_redundant_regions.html\" name=\"main\">\n". 
		"</FRAMESET>\n".
		"</HTML>";
	
	    close REDUNDANT_REGIONS_INDEX;
	}
    }

    #	
    #make summary file
    #
	
    if (@redundant_repeat_summary_file){
	open (REDUNDANT_REPEAT_REGIONS,">$output_prefix\_redundant_regions.html") 
	    or die "cannot open $output_prefix\_redundant_region.txt";
	
	print REDUNDANT_REPEAT_REGIONS "<BODY bgcolor=\"\#FFFFCC\">\n".
				       "<TABLE BORDER=3 CELLSPACING=1 CELLPADDING=1><TR>".
				       "<TD WIDTH=80 BGCOLOR=\"\#FFCC66\"><CENTER>Sequence Index<\/CENTER><\/TD>".
				       "<TD WIDTH=400><CENTER>Sequence Description<\/CENTER><\/TD>".
				       "<\/TR>\n".
				       "@redundant_repeat_summary_file".
				       "</TABLE>\n".
	  			       "</BODY></HTML>\n";
    }
    
    return $total_size_of_repeat_regions;
}

sub all_divisores{
    my $int = shift;
    my @result = (1);
    my $limit = $int /2;
    for(my $i=2; $i <= $limit;$i++){
	if (!($int%$i)){
	    push(@result,$i);
	}
    }
    return @result;
}

